import prisma from "../client";
import { Prisma } from "@prisma/client";
import { LoanCreateData, LoanCreateResult } from "./types/data";
import { Result } from "@badrap/result";

export const create = async (data: LoanCreateData): LoanCreateResult => {
  try {
    return Result.ok(
      await prisma.$transaction(
        async (tx) => {
          const user = await tx.member.findUniqueOrThrow({
            where: {
              email: data.user,
            },
            select: {
              id: true,
            },
          });

          const book = await tx.book.findFirstOrThrow({
            where: {
              title: data.book,
            },
            select: {
              id: true,
            },
          });

          const isBookLoaned = await tx.loan.findFirst({
            where: {
              bookId: book.id,
              returnDate: null,
            },
          });

          if (isBookLoaned) {
            throw new Error("Book is currently loaned");
          }

          const newLoan = await tx.loan.create({
            data: {
              bookId: book.id,
              memberId: user.id,
              loanDate: new Date(),
            },
          });
          return newLoan;
        },
        {
          isolationLevel: Prisma.TransactionIsolationLevel.Serializable,
        }
      )
    );
  } catch (error) {
    return Result.err(error as Error);
  }
};
